/**
 * Generated by Apache Royale Compiler from org/apache/royale/net/URLStream.as
 * org.apache.royale.net.URLStream
 *
 * @fileoverview
 *
 * @suppress {checkTypes|accessControls}
 */

goog.provide('org.apache.royale.net.URLStream');

goog.require('org.apache.royale.events.DetailEvent');
goog.require('org.apache.royale.events.Event');
goog.require('org.apache.royale.events.EventDispatcher');
goog.require('org.apache.royale.events.ProgressEvent');
goog.require('org.apache.royale.net.HTTPHeader');
goog.require('org.apache.royale.net.URLRequest');
goog.require('org.apache.royale.net.URLRequestHeader');
goog.require('org.apache.royale.net.utils.encodeAsQueryString');
goog.require('org.apache.royale.utils.BinaryData');
goog.require('org.apache.royale.utils.Endian');
goog.require('org.apache.royale.utils.Language');



/**
 * constructor
 * @constructor
 * @extends {org.apache.royale.events.EventDispatcher}
 */
org.apache.royale.net.URLStream = function() {
  
  this.endian = org.apache.royale.utils.Endian.BIG_ENDIAN;
  org.apache.royale.net.URLStream.base(this, 'constructor');
};
goog.inherits(org.apache.royale.net.URLStream, org.apache.royale.events.EventDispatcher);


/**
 * @protected
 * @type {XMLHttpRequest}
 */
org.apache.royale.net.URLStream.prototype.xhr;


/**
 * @private
 * @type {Function}
 */
org.apache.royale.net.URLStream._corsCredentialsChecker;


/**
 * Intended as global configuration of CORS withCredentials setting on requests
 * This method is not reflectable, is js-only and is eliminated via dead-code-elimination
 * in js-release builds if it is never used.
 * URLStream is used a service base in other service classes, so this provides
 * a 'low level' solution for a bead that can work at application level.
 * The 'checker' function parameter should be a function that takes a url as its single argument
 * and returns true or false depending on whether 'withCredentials' should be set for
 * that http request. Set it to null to always be false.
 * @asprivate
 * @royalesuppressexport
 * @param {Function} checker
 */
org.apache.royale.net.URLStream.setCORSCredentialsChecker = function(checker) {
  org.apache.royale.net.URLStream._corsCredentialsChecker = checker;
};


/**
 * @type {number}
 */
org.apache.royale.net.URLStream.prototype.bytesLoaded = 0;


/**
 * @type {number}
 */
org.apache.royale.net.URLStream.prototype.bytesTotal = 0;


/**
 * loads the request
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 *  @royaleignorecoercion org.apache.royale.utils.BinaryData
 * @param {org.apache.royale.net.URLRequest} urlRequest
 */
org.apache.royale.net.URLStream.prototype.load = function(urlRequest) {
  this.requestStatus = 0;
  this.createXmlHttpRequest();
  if (org.apache.royale.net.URLStream._corsCredentialsChecker != null) {
    this.xhr.withCredentials = !!(org.apache.royale.net.URLStream._corsCredentialsChecker(urlRequest.url));
  }
  this.xhr.open(urlRequest.method, urlRequest.url);
  this.xhr.responseType = "arraybuffer";
  this.xhr.addEventListener("readystatechange", org.apache.royale.utils.Language.closure(this.org_apache_royale_net_URLStream_xhr_onreadystatechange, this, 'org_apache_royale_net_URLStream_xhr_onreadystatechange'), false);
  this.xhr.addEventListener("progress", org.apache.royale.utils.Language.closure(this.org_apache_royale_net_URLStream_xhr_progress, this, 'org_apache_royale_net_URLStream_xhr_progress'), false);
  var /** @type {string} */ contentType;
  for (var /** @type {number} */ i = 0; i < urlRequest.requestHeaders.length; i++) {
    var /** @type {org.apache.royale.net.URLRequestHeader} */ header = urlRequest.requestHeaders[i];
    if (header.name.toLowerCase() == "content-type") {
      contentType = header.value;
    }
    this.xhr.setRequestHeader(header.name, header.value);
  }
  if (!contentType && urlRequest.contentType) {
    contentType = urlRequest.contentType;
    this.xhr.setRequestHeader("Content-type", urlRequest.contentType);
  }
  var /** @type {Object} */ requestData;
  if (org.apache.royale.utils.Language.is(urlRequest.data, org.apache.royale.utils.BinaryData)) {
    requestData = urlRequest.data.data;
  } else if (contentType == org.apache.royale.net.HTTPHeader.APPLICATION_JSON) {
    if (org.apache.royale.utils.Language.is(urlRequest.data, String)) {
      requestData = urlRequest.data;
    } else if (urlRequest.data) {
      requestData = JSON.stringify(urlRequest.data);
    } else {
      requestData = "";
    }
  } else if (org.apache.royale.utils.Language.is(urlRequest.data, FormData)) {
    requestData = urlRequest.data;
  } else if (org.apache.royale.utils.Language.is(urlRequest.data, String)) {
    requestData = urlRequest.data;
  } else {
    requestData = org.apache.royale.net.utils.encodeAsQueryString(urlRequest.data);
  }
  this.send(requestData);
  
};


/**
 * Returns the string value of a specific response header.
 * The header name is case-insensitive.
 * 
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.9.11
 * @param {string} header
 * @return {string}
 */
org.apache.royale.net.URLStream.prototype.getResponseHeader = function(header) {
  
  return this.xhr.getResponseHeader(header);
};


/**
 * @protected
 */
org.apache.royale.net.URLStream.prototype.createXmlHttpRequest = function() {
  this.xhr = new XMLHttpRequest();
};


/**
 * send is a protected function in js so a subclass can attach an upload listener
 * without rewriting the whole load() function
 * @protected
 * @param {Object} requestData
 */
org.apache.royale.net.URLStream.prototype.send = function(requestData) {
  this.xhr.send(requestData);
};


/**
 *  Download is progressing (JS only).
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @private
 * @param {Object} error
 */
org.apache.royale.net.URLStream.prototype.org_apache_royale_net_URLStream_xhr_progress = function(error) {
  var /** @type {org.apache.royale.events.ProgressEvent} */ progEv = new org.apache.royale.events.ProgressEvent(org.apache.royale.events.ProgressEvent.PROGRESS);
  progEv.current = this.bytesLoaded = (error.loaded) >>> 0;
  progEv.total = this.bytesTotal = (error.total) >>> 0;
  this.dispatchEvent(progEv);
  if (this.onProgress)
    this.onProgress(this);
};


/**
 *  HTTP status change (JS only).
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @private
 * @param {*} error
 */
org.apache.royale.net.URLStream.prototype.org_apache_royale_net_URLStream_xhr_onreadystatechange = function(error) {
  this.org_apache_royale_net_URLStream_setStatus((this.xhr.status) >> 0);
  if (this.xhr.readyState != 4)
    return;
  if (this.xhr.status == 0) {
    this.dispatchEvent(new org.apache.royale.events.DetailEvent("communicationError"));
    if (this.onError)
      this.onError(this);
  } else if (this.xhr.status < 200) {
    this.dispatchEvent(new org.apache.royale.events.DetailEvent("communicationError", false, false, "" + this.requestStatus));
    if (this.onError)
      this.onError(this);
  } else if (this.xhr.status < 300) {
    this.dispatchEvent(new org.apache.royale.events.Event("complete"));
    if (this.onComplete)
      this.onComplete(this);
  } else {
    this.dispatchEvent(new org.apache.royale.events.DetailEvent("communicationError", false, false, "" + this.requestStatus));
    if (this.onError)
      this.onError(this);
  }
  this.cleanupCallbacks();
};


/**
 *  Set the HTTP request status.
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @private
 * @param {number} value
 */
org.apache.royale.net.URLStream.prototype.org_apache_royale_net_URLStream_setStatus = function(value) {
  if (value != this.requestStatus) {
    this.requestStatus = value;
    this.dispatchEvent(new org.apache.royale.events.DetailEvent("httpStatus", false, false, "" + value));
    if (this.onStatus)
      this.onStatus(this);
  }
};


/**
 *  Abort an connection.
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 */
org.apache.royale.net.URLStream.prototype.close = function() {
  
  this.xhr.abort();
  this.cleanupCallbacks();
};


/**
 * @type {number}
 */
org.apache.royale.net.URLStream.prototype.requestStatus = 0;


/**
 * @type {string}
 */
org.apache.royale.net.URLStream.prototype.endian;


/**
 *  Cleanup all callbacks.
 *
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @protected
 */
org.apache.royale.net.URLStream.prototype.cleanupCallbacks = function() {
  this.onComplete = null;
  this.onError = null;
  this.onProgress = null;
  this.onStatus = null;
};


/**
 * @type {Function}
 */
org.apache.royale.net.URLStream.prototype.onComplete;


/**
 * @type {Function}
 */
org.apache.royale.net.URLStream.prototype.onError;


/**
 * @type {Function}
 */
org.apache.royale.net.URLStream.prototype.onProgress;


/**
 * @type {Function}
 */
org.apache.royale.net.URLStream.prototype.onStatus;


/**
 *  Convenience function for complete event to allow chaining.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @param {Function} callback
 * @return {org.apache.royale.net.URLStream}
 */
org.apache.royale.net.URLStream.prototype.complete = function(callback) {
  this.onComplete = callback;
  return this;
};


/**
 *  Convenience function for error event to allow chaining.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @param {Function} callback
 * @return {org.apache.royale.net.URLStream}
 */
org.apache.royale.net.URLStream.prototype.error = function(callback) {
  this.onError = callback;
  return this;
};


/**
 *  Convenience function for progress event to allow chaining.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @param {Function} callback
 * @return {org.apache.royale.net.URLStream}
 */
org.apache.royale.net.URLStream.prototype.progress = function(callback) {
  this.onProgress = callback;
  return this;
};


/**
 *  Convenience function for status event to allow chaining.
 *  
 *  @langversion 3.0
 *  @playerversion Flash 10.2
 *  @playerversion AIR 2.6
 *  @productversion Royale 0.7.0
 * @param {Function} callback
 * @return {org.apache.royale.net.URLStream}
 */
org.apache.royale.net.URLStream.prototype.status = function(callback) {
  this.onStatus = callback;
  return this;
};


/**
 * @nocollapse
 * @export
 * @type {org.apache.royale.utils.BinaryData}
 */
org.apache.royale.net.URLStream.prototype.response;


org.apache.royale.net.URLStream.prototype.get__response = function() {
  return new org.apache.royale.utils.BinaryData(this.xhr.response);
  
};


Object.defineProperties(org.apache.royale.net.URLStream.prototype, /** @lends {org.apache.royale.net.URLStream.prototype} */ {
/**
 * @type {org.apache.royale.utils.BinaryData}
 */
response: {
get: org.apache.royale.net.URLStream.prototype.get__response}}
);


/**
 * Metadata
 *
 * @type {Object.<string, Array.<Object>>}
 */
org.apache.royale.net.URLStream.prototype.ROYALE_CLASS_INFO = { names: [{ name: 'URLStream', qName: 'org.apache.royale.net.URLStream', kind: 'class' }] };



/**
 * Reflection
 *
 * @return {Object.<string, Function>}
 */
org.apache.royale.net.URLStream.prototype.ROYALE_REFLECTION_INFO = function () {
  return {
    variables: function () {
      return {
        'bytesLoaded': { type: 'uint', get_set: function (/** org.apache.royale.net.URLStream */ inst, /** * */ v) {return v !== undefined ? inst.bytesLoaded = v : inst.bytesLoaded;}},
        'bytesTotal': { type: 'uint', get_set: function (/** org.apache.royale.net.URLStream */ inst, /** * */ v) {return v !== undefined ? inst.bytesTotal = v : inst.bytesTotal;}},
        'requestStatus': { type: 'int', get_set: function (/** org.apache.royale.net.URLStream */ inst, /** * */ v) {return v !== undefined ? inst.requestStatus = v : inst.requestStatus;}},
        'endian': { type: 'String', get_set: function (/** org.apache.royale.net.URLStream */ inst, /** * */ v) {return v !== undefined ? inst.endian = v : inst.endian;}},
        'onComplete': { type: 'Function', get_set: function (/** org.apache.royale.net.URLStream */ inst, /** * */ v) {return v !== undefined ? inst.onComplete = v : inst.onComplete;}},
        'onError': { type: 'Function', get_set: function (/** org.apache.royale.net.URLStream */ inst, /** * */ v) {return v !== undefined ? inst.onError = v : inst.onError;}},
        'onProgress': { type: 'Function', get_set: function (/** org.apache.royale.net.URLStream */ inst, /** * */ v) {return v !== undefined ? inst.onProgress = v : inst.onProgress;}},
        'onStatus': { type: 'Function', get_set: function (/** org.apache.royale.net.URLStream */ inst, /** * */ v) {return v !== undefined ? inst.onStatus = v : inst.onStatus;}}
      };
    },
    accessors: function () {
      return {
        'response': { type: 'org.apache.royale.utils.BinaryData', access: 'readonly', declaredBy: 'org.apache.royale.net.URLStream'}
      };
    },
    methods: function () {
      return {
        'URLStream': { type: '', declaredBy: 'org.apache.royale.net.URLStream'},
        'load': { type: 'void', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [ 'org.apache.royale.net.URLRequest', false ]; }},
        'getResponseHeader': { type: 'String', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [ 'String', false ]; }},
        'close': { type: 'void', declaredBy: 'org.apache.royale.net.URLStream'},
        'complete': { type: 'org.apache.royale.net.URLStream', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [ 'Function', false ]; }},
        'error': { type: 'org.apache.royale.net.URLStream', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [ 'Function', false ]; }},
        'progress': { type: 'org.apache.royale.net.URLStream', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [ 'Function', false ]; }},
        'status': { type: 'org.apache.royale.net.URLStream', declaredBy: 'org.apache.royale.net.URLStream', parameters: function () { return [ 'Function', false ]; }}
      };
    }
  };
};
/**
 * @const
 * @type {number}
 */
org.apache.royale.net.URLStream.prototype.ROYALE_COMPILE_FLAGS = 10;